/**
 * libcsock - library for managed UNIX socket implementation
 * 1999-2000 by Christophe Lucas <c.lucas@ifrance.com>
 *
 * This file is subject to the terms and conditions of the GNU General
 * Public License.  See the file COPYING in the main directory of the
 * libcsock distribution for more details.
 *****************************************************************************
 * csock.c
 ***************************************************************************** 
 * v 0.0.1 - 23/07/00 - christophe, lucas  - premire dition
 * v 0.0.2 - 24/04/01 - christophe, lucas  - rvision
 * v 0.0.3 - 30/04/01 - c.lucas && a.pique - added write_n and read_n 
 * v 0.0.4 - 09/05/01 - c.lucas		   - bug fixes on write_n and read_n
 ***************************************************************************** 
 */

#include "csock.h"

/**
 * errno : variable permettant de grer les erreurs survenus pendant 
 * l'xcution du programme utilisant cette librairie
 */
extern int errno;

/**
 * error_socket: permet l'affichage de message d'erreurs variables
 * PARAMETRES:
 * - format : chaine de format
 */
void error_socket( const char* format , ... )
{
    va_list args;
    va_start(args , format );
    vfprintf(stderr , format , args );
    va_end(args);
    exit(1);
}
    
/**
 * creat_socket: permet de crer un socket suivant les informations qui
 * sont passs en paramtres  cette fonction
 * PARAMETRES:
 * - service : dfinit sur quel port va tre active ou passive cet socket
 * - transport : type de transport (UDP ou TCP)
 * - sin : structure renfermant les diffrentes informations au sujet du socket
 * RETOUR:
 * - int : entier identidiant le descripteur du socket
 */    
int creat_socket( const char* service , const char* transport , struct sockaddr_in* sin )
{
    /*
     * Structures permettant de rcuprer des infos numriques
     *  partir des noms de service (nport) et protocole
     */
    struct servent* iservice;
    struct protoent* itransport;
    int s , type , proto ;
    
    /*Initialisation de la structure socket*/
    memset( sin , 0 , sizeof(*sin) ) ;
    sin->sin_family = AF_INET ;
    
    /* Recherche du numro de port */
    sin->sin_port = htons( atoi(service) ) ;
    if( !sin->sin_port ) {
	/*Si c'est pas un numero de port on recherche dans /etc/services */
	iservice=getservbyname(service , transport );
	if( !iservice ) {
	    error_socket("Service non enregistr: %s",service ); 
	}
	sin->sin_port = iservice ->s_port;
    }
    
    /* Recherche du numro de protocole */
    itransport = getprotobyname(transport);
    if( !itransport )
    {
	error_socket("Protocole non enregistr: %s",transport);
    }
    proto = itransport->p_proto ;
    
    /* Diffrenciation entre "udp" et "tcp" */
    if( !strcmp( transport , "udp" ) )
	type = SOCK_DGRAM ; // UDP
    else
	type = SOCK_STREAM; // TCP
    
    s = socket( AF_INET, type , proto );
    if( s < 0 )
	error_socket("Creation de socket impossible: %s",strerror(errno) );
    
    return s;
    
}/*creat_socket*/

/**
 * client_socket :
 * permet la Cration d'une socket pour un client suivant les informations
 * que l'on passe  cette fonction.
 * PARAMETRES:
 * - host: spcifie  quel serveur, nous voulons tre sonnect 
 *   (chaine de caractre ou bien adresse IPv4 de type point).
 * - service : dfinit sur quel port va tre active ou passive cet socket
 * - transport : type de transport (UDP ou TCP)
 * RETOUR:
 * - int : entier identifiant le descripteur du socket client
 */
int client_socket( const char* host , const char* service , const char* transport )
{
    /*strusture dcrivant la socket*/
    struct sockaddr_in sin;
    struct hostent* ihost;
    
    int s;
    
    /*cration d'une socket*/
    s = creat_socket(service , transport , &sin );
    
    /*Recherche de l'adresse IP de host*/
    sin.sin_addr.s_addr = inet_addr(host) ;
    
    if( sin.sin_addr.s_addr == INADDR_NONE ) {
	/*
	 * Si host est un nom et non pas une addr IP
	 * On recherche dans /etc/hosts et dns
	 */
	ihost = gethostbyname( host );
	if( !ihost )
	    error_socket("Hte inconnu:%s",host );
	    
	memcpy(&sin.sin_addr , ihost->h_addr , ihost->h_length );
    }
    
    /*connexion de la socket*/
    if( connect( s , (struct sockaddr* ) &sin , sizeof(sin) ) < 0 )
	return -1;
	//error_socket("Impossible de se connecter  %s[%s] : %s" , host, service , strerror(errno) );
	
    return s;	
}

/**
 * server_socket:
 * permet la cration d'une socket de type serveur suivant les informations
 * que l'on passe  cette fonction ; permettant d'acceuillir lqueue connection(s).
 * PARAMETRES:
 * - service : dfinit sur quel port va tre active ou passive cet socket
 * - transport : type de transport (UDP ou TCP)
 * - lqueue : nombre de connection(s) admissible(s) par ce serveur
 * RETOUR:
 * - int : entier identifiant le descripteur du socket serveur
 */
int server_socket( const char* service , const char* transport , int lqueue )
{
    struct sockaddr_in sin;
    
    int s;
    
    s = creat_socket( service , transport , &sin );
    
    sin.sin_addr.s_addr = INADDR_ANY ;
     
    /*Allocation (affectation) du numro de port*/
    if( bind( s , (struct sockaddr*) &sin , sizeof( sin ) ) < 0 )
    {
	error_socket("Impossible de lier la socket  %s: %s",service , strerror(errno) );
    }
    
    /* on place la socket en attente si transport n'est pas udp */
    if( strcmp(transport, "udp" ) && listen( s , lqueue )<0 )
	error_socket("Mode passif impossible sur %s: %s",service,strerror(errno) );
    
    return s;
}/* server_socket*/

/**
 * accepter :
 * accepter permet d'accepter une connection lorsque l'on est en mode connect,
 * et de ne pas renvoyer quelque chose en cas d'iterruption logicielle.
 * PARAMETRES:
 * - fd : socket d'coute
 * - st : structure permettant de stocker les informations sur la connections qui va tre accpeter.
 * RETOUR:
 * - int : entier identifiant le socket de la connection accueillie.
 */
int accepter( int fd , struct sockaddr_in* st )
{
	int newfd;
	int lg = sizeof( struct sockaddr_in );

	do {
		newfd = accept( fd , (struct sockaddr*) &st , &lg );
	}while( newfd == -1 && (errno==EMFILE ) );
		
	if( errno == EINTR ) {
		newfd = -2 ;
	}

return newfd;
}

/**
 * write_n :
 * write_n permet d'crire "effectivement" nb octets sur 
 * un file descriptor (file,socket,...) 
 * PARAMETRES:
 * - fd : socket sur lequel on doit crire
 * - p  : pointeur contenant la chaine de caractre  crire
 * - nb : nombre d'octet(s)  crire
 * RETOUR:
 * - int : entier donnant le nombre d'octets crits ou -1 en cas d'erreur
 */
int write_n(register int fd , void * p , int nb )
{
  int nbleft,nbwritten;
  nbleft=nb;

  while(nbleft>0){
    nbwritten = write(fd,p,nbleft);
    if(nbwritten<0){
      switch(errno){
      case EINTR : nbwritten = 0;
      case EAGAIN : nbwritten = 0;
      default : return -1;
      }
    }
    nbleft-=nbwritten;
    p+=nbwritten;
  }
  return nb;
}

/**
 * read_n :
 * read_n permet de lire "effectivement" nb octets sur 
 * un file descriptor (file,socket,...) 
 * PARAMETRES:
 * - fd : socket sur lequel on doit lire
 * - p  : pointeur servant  contenir ce qui va tre lu
 * - nb : nombre d'octet(s)  crire
 * RETOUR:
 * - int : entier donnant le nombre d'octets lus ou -1 en cas d'erreur
 */

int read_n(register int fd, void *p , int nb)
{
  int nbleft,nbread;
  nbleft=nb;

  while(nbleft>0){
    nbread = read(fd,p,nbleft);
    if(nbread<0){
      switch(errno){
      case EINTR : nbread = 0;
      default : return(-1);
      }
    }
    nbleft-=nbread;
    p+=nbread;
  }
  return nb;
}
